/*
 * Choeps Next Generation
 * 
 * Mark Spencer <markster@marko.net>
 *
 * Copyright(C) 1999, Adtran, Inc.
 * 
 * Distributed under the terms of the GNU General Public License (GPL) Version
 *
 * Cheops Agent Main File
 *
 */

#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include "cheops.h"
#include "logger.h"
#include "event.h"
#include "agent-discover.h"
#include "agent-settings.h"
#include "agent-osscan.h"

#ifdef DEBUG_CHEOPS_AGENT
#define DEBUG DEBUG_M
#else
#define DEBUG(a)
#endif

static struct handler {
	int id;
	cheops_event_handler cb;
	char *desc;
} agent_handlers[];


/* Buffer for creating all events */
char ebuf[MAX_EVENT_SIZE];
event_hdr *eh = (event_hdr *)ebuf;
event *ee = (event *)(ebuf + sizeof(event_hdr));

struct handler agent_handlers[] = {
	{ EVENT_OS_SCAN,       handle_osscan_request,   "Os-Scan Request Handler" },
	{ EVENT_DISCOVER_IPV4, handle_discover_request, "Discover Request Handler" },
	{ EVENT_SET_SETTINGS,  handle_set_settings, "Set Agent Settings Handler" },
//	{ EVENT_DNS_QUERY,     handle_dns_query,    "DNS Query Handler" },
	{ -1 },
};

void usage()
{
	fprintf(stderr, "Usage: cheops-agent [-n|-l|-a]\n"
	                "          -n: Listen using network method\n"
			"          -l: Listen using local method\n"
			"          -f: Fork to background\n"
			"          -a: Listen using all methods (default)\n");
	exit(1);
}

void register_agent_handlers()
{
	struct handler *h = agent_handlers;
	while(h->id > -1) {
		/* Try to register each of our shell items, but don't replace an existing
		   handler */
		if (event_register_handler(h->id, h->cb, 0)) 
			clog(LOG_WARNING, "Unable to register handler for '%s'\n", h->desc);
		h++;
	}
}

void *do_it(void *type)
{
	if (event_create_agent((int)type)) {
		clog(LOG_ERROR, "unable to listen.\n");
		exit(1);
	}
	register_agent_handlers();
	cheops_main();
	return NULL;
}


static void myexit(int sig)
{
    exit(0);
}

void do_tasks(int type)
{
	do_it((void *)type);
}

int main(int argc, char *argv[])
{
	char c;
	int local=0;
	int ipv4=0;
	int all=0;
	int dofork=0;
	pid_t pid;
	int type;
	
	if(getuid())
	{
		printf("You have to be su to run %s\n\n",argv[0]);
		myexit(0);
	}
	
	while((c = getopt(argc, argv, "alnf")) >=0) {
		switch(c) {
		case 'a':
			all++;
			break;
		case 'n':
			ipv4++;
			break;
		case 'l':
			local++;
			break;
		case 'f':
			dofork++;
			break;
		default:
			usage();
		}
	}

	signal(SIGPIPE, SIG_IGN);
	signal(SIGINT,  myexit);
	signal(SIGTERM, myexit);

	/* Check for garbage */
	if (optind != argc)
		usage();
	/* Default is to listen on  all methods */
	if (!all && !local & !ipv4) 
		all++;

	type = (all || local) ? AGENT_TYPE_LOCAL : 0; 
	type |= (all || ipv4) ? AGENT_TYPE_IPV4 : 0 ;

	if (dofork) {

		switch((pid = fork())) {
		case 0:
			/* Child */
			/* Main thread */
			do_tasks(type);
			break;
		case -1:
			/* Error */
			clog(LOG_ERROR, "fork() failed: %s\n", strerror(errno));
			exit(1);
			break;
		default:
			/* Parent */
			DEBUG(clog(LOG_NOTICE, "cheops-agent launched, pid %d\n", pid));
		}
	}
	else
		do_tasks(type);

	myexit(0);
	return(1);
}

