/*
 * Choeps Next Generation GUI
 * 
 * Brent Priddy <toopriddy@mailcity.com>
 *
 * Copyright(C) 1999, Adtran, Inc.
 * 
 * Distributed under the terms of the GNU General Public License (GPL) Version
 *
 * Config file reader and parser
 *
 */

#include <stdio.h>
#include <gnome.h>
#include "event.h"
#include "logger.h"
#include "gui-sched.h"
#include "cheops-gui.h"
#include "gui-viewspace.h"
#include "gui-handlers.h"
#include "gui-settings.h"
#include "gui-utils.h"
#include "gui-canvas.h"
#include "gui-config.h"
#include "gui-service.h"
#include "ip_utils.h"
#include "gui-pixmap.h"
		
void add_name_to_buf(char *base, char *buf, char *add)
{
	sprintf(buf, "%s%s",base,add);
}

char *config_convert_notes(char *notes)
{
	char *string;
	int len = strlen(notes);
	int i, index;
	
	if( ( string = malloc(len) ) == NULL)
	{
		printf("\n bad error\n");
		exit(1);
	}
	
	index = 0;
	for(i = 0; i <= len; i++)
	{
		if((string[i] = notes[i + index]) == '%' && 
				notes[i + index + 1] == '2' && 
				notes[i + index + 2] == '0' 
		)
		{
			string[i] = '\n';
			index +=2;
		}
	}
	return(string);
}	

char *config_get_agent_ip_address(void)
{
	int def;
	char *ip;
	char buf[256];
	char buf2[256];
	FILE *fp;
	
	sprintf(buf2,"%s/%s",getenv("HOME"), CHEOPS_DEFAULT_FILENAME_NOT_HOME);
	
	if( (fp = fopen(buf2, "rt")) == NULL)
	{
		return(NULL);
	}
	fclose(fp);
	
	sprintf(buf,"=%s=/global/agent_ip=NO_IP",buf2);
	
	ip = gnome_config_get_string_with_default(buf,&def);
	if(def)
	{
		free(ip);
		ip = NULL;
	}
	
	return(ip);	
}

void read_config_file(GtkWidget *w, GtkWidget *fs)
{
	char buf[256], buf2[256], base[256], *filename;
	net_page *np;
	page_object *po;
	int i=0, j=0, def=0;
	char *name;
	char *po_name;
	char *po_ip;
	char *fp_string;
	char *fp_pixmap;
	char *service_name;
	int service_port;
	int service_protocol;
	char *service_command;
	
// default the service list and pixmaps list so the use without a config file can use these
	os_pixmap_list_default();
	service_list_default();

	if(fs != NULL)
	{
		filename = makestring(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));

		if(main_window->filename)
			free(main_window->filename);
			
		main_window->filename = filename;
	
		gtk_widget_destroy(fs);
	}
	else
	{
		filename = main_window->filename;
	}

// this is the case when we have started 'aclean and have no config file
	if(filename == NULL)
		return;
	
	/* lets write somethings in it */
	sprintf(buf,"=%s=/global/discover_retries=2",filename);
	options_discover_retries = gnome_config_get_int_with_default(buf,&def);
	apply_the_setting(OPTIONS_DISCOVER_RETRIES);

	sprintf(buf,"=%s=/global/options_operating_system_check=%d",filename, options_operating_system_check);
	options_operating_system_check = gnome_config_get_int_with_default(buf,&def);
	apply_the_setting(OPTIONS_OPERATING_SYSTEM_CHECK);

	sprintf(buf,"=%s=/global/options_confirm_delete=%d",filename, options_confirm_delete);
	options_confirm_delete = gnome_config_get_int_with_default(buf,&def);
	apply_the_setting(OPTIONS_CONFIRM_DELETE);

	sprintf(buf,"=%s=/global/options_save_changes_on_exit=%d",filename, options_save_changes_on_exit);
	options_save_changes_on_exit = gnome_config_get_int_with_default(buf,&def);
	apply_the_setting(OPTIONS_SAVE_CHANGES_ON_EXIT);

	sprintf(buf,"=%s=/global/options_reverse_dns=%d",filename, options_reverse_dns);
	options_reverse_dns = gnome_config_get_int_with_default(buf,&def);
	apply_the_setting(OPTIONS_REVERSE_DNS);

	sprintf(buf,"=%s=/global/options_use_ip_for_label=%d",filename, options_use_ip_for_label);
	options_use_ip_for_label = gnome_config_get_int_with_default(buf,&def);
	apply_the_setting(OPTIONS_USE_IP_FOR_LABEL);

/*
 * These are all of the strings to match OS names with
 */
 	for(i = 0;;)
 	{
		sprintf(buf,"=%s=/global/fingerprint_string%d=NO_STRING",filename,i);
		fp_string = gnome_config_get_string_with_default(buf,&def);
		if(def)
		{
			free(fp_string);
			break;
		}
			
		sprintf(buf,"=%s=/global/fingerprint_pixmap%d=NO_STRING",filename,i);
		fp_pixmap = gnome_config_get_string_with_default(buf,&def);
		i++;
		
		if(def)
		{
			free(fp_string);
			free(fp_pixmap);
			break;
		}
			
		os_pixmap_list_add(fp_string, fp_pixmap);
		free(fp_string);
		free(fp_pixmap);
	}

/*
 * These are all of the strings for the type of services and their command
 */
 	for(i = 0;;)
 	{
		sprintf(buf,"=%s=/global/service%d_name=NO_STRING",filename,i);
		service_name = gnome_config_get_string_with_default(buf,&def);
		if(def)
		{
			free(service_name);
			break;
		}
			
		sprintf(buf,"=%s=/global/service%d_port=0",filename,i);
		service_port = gnome_config_get_int_with_default(buf,&def);
		
		if(def)
		{
			free(service_name);
			break;
		}
			
		sprintf(buf,"=%s=/global/service%d_protocol=0",filename,i);
		service_protocol = gnome_config_get_int_with_default(buf,&def);
		
		if(def)
		{
			free(service_name);
			break;
		}
			
		sprintf(buf,"=%s=/global/service%d_command=NO_STRING",filename,i);
		service_command = gnome_config_get_string_with_default(buf,&def);
		
		if(def)
		{
			free(service_name);
			free(service_command);
			break;
		}
			
		i++;
		service_list_add(service_name, service_port, service_protocol, service_command);
		free(service_name);
		free(service_command);
	}


//	sprintf(buf,"=%s=/global/tooltips_timeout=1",filename);
//	tooltips_timeout = gnome_config_get_int_with_default(buf,&def);
	
	for(i=0;;i++)
	{
		sprintf(base,"=%s=/viewspace%d",filename,i);
		
		sprintf(buf,"%s%s",base, "/name=NOTHING");
		name = gnome_config_get_string_with_default(buf, &def);
				
		if(def) /* if it uses the default then there are no more viewspaces in the config file */
		{
			free(name);
			break;
		}
		np = make_viewspace(name);
		
		while(gtk_events_pending())		/* have to let the notebook add the page */
			gtk_main_iteration();
                                                       			
				
		/* add the reference to the page_objects in the netpage sections */
		strcat(base,"/page_object_");

		for(j=0;;j++)
		{
			sprintf(buf, "%s%d",base,j);

			add_name_to_buf(buf, buf2, "_ip=NOTHING");
			po_ip = gnome_config_get_string_with_default(buf2,&def);
			if(def)
			{
				free(po_ip);
				break;
			}

			add_name_to_buf(buf, buf2, "_name=NOTHING");
			po_name = gnome_config_get_string_with_default(buf2,&def);
			if(def)
			{
				free(po_name);
				po_name = NULL;
			}
			po = add_host_entry_to_net_page(np,str2ip(po_ip),po_name);

			add_name_to_buf(buf, buf2, "_notes=NOTHING");
			po->notes = gnome_config_get_string_with_default(buf2,&def);
			
			if(def)
			{
				free(po->notes);
				po->notes = NULL;
			}
			else
				po->notes = config_convert_notes(po->notes);
				
			add_name_to_buf(buf, buf2, "_x=0");
			po->x = gnome_config_get_int_with_default(buf2,&def);
			
			add_name_to_buf(buf, buf2, "_y=0");
			po->y = gnome_config_get_int_with_default(buf2,&def);

// There is no need to save the flags right now			
//			add_name_to_buf(buf, buf2, "_flags=0");
//			po->flags = gnome_config_get_int_with_default(buf2,&def);
			
			po->flags |= PAGE_OBJECT_CONFIG_NEW;
			page_object_display(po);
			
			//start the discovery
			do_discover(primary_agent, ip2str(po->ip), np, 0);
		}

	}
}

char *page_object_convert_notes(page_object *po)
{
	char *string;
	int len = strlen(po->notes);
	int i,count, index;
	
	for(i=0,count=0;i<len;i++)
	{
		if(po->notes[i] == '\n')
			count++;
	}
	
	if( (string = malloc(strlen(po->notes) + 1 + count*2)) == NULL)
	{
		printf("\n bad error\n");
		exit(1);
	}
	
	if(count == 0)
		return(string);
	
	index = 0;
	for(i = 0; i <= len; i++)
	{
		if((string[i + index] = po->notes[i]) == '\n')
		{
			string[i+ index] = '%';
			string[i+ index + 1] = '2';
			string[i+ index + 2] = '0';
			index +=2;
		}
	}
	return(string);
}	

void write_config_file(GtkWidget *w, GtkWidget *fs)
{
	char buf[256], buf2[256], base[256], *filename;
	net_page *np;
	page_object *po;
	int i=0,j=0;
	FILE *fp;
	os_pixmap_list_t *pos;
	service_list_t *srv;
		
	if(fs != NULL)
	{
		filename = makestring(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));

		if(main_window->filename)
			free(main_window->filename);
			
		main_window->filename = filename;
	
		gtk_widget_destroy(fs);
	}
	else
	{
		filename = main_window->filename;
	}
	
	/* clear out the destination file if it is there */
	fp = fopen(filename,"wt");
	fclose(fp);
	
	/* lets write something in it */
	if(main_window->agent_ip)
	{
		sprintf(buf,"=%s=/global/agent_ip",filename);
		gnome_config_set_string(buf,main_window->agent_ip);
	}

	sprintf(buf,"=%s=/global/options_discover_retries",filename);
	gnome_config_set_int(buf,options_discover_retries);

	sprintf(buf,"=%s=/global/options_operating_system_check",filename);
	gnome_config_set_int(buf,options_operating_system_check);

	sprintf(buf,"=%s=/global/options_confirm_delete",filename);
	gnome_config_set_int(buf,options_confirm_delete);

	sprintf(buf,"=%s=/global/options_reverse_dns",filename);
	gnome_config_set_int(buf,options_reverse_dns);

	sprintf(buf,"=%s=/global/options_save_changes_on_exit",filename);
	gnome_config_set_int(buf,options_save_changes_on_exit);

	sprintf(buf,"=%s=/global/options_use_ip_for_label",filename);
	gnome_config_set_int(buf,options_use_ip_for_label);

//	sprintf(buf,"=%s=/global/tooltips_timeout",filename);
//	gnome_config_set_int(buf,options_tooltips_timeout);

/*
 * These are all of the strings to match OS names with
 */
 	for(i = 0, pos = os_pixmap_list; pos; pos = pos->next, i++)
 	{
		sprintf(buf,"=%s=/global/fingerprint_string%d",filename,i);
		gnome_config_set_string(buf,pos->string);
			
		sprintf(buf,"=%s=/global/fingerprint_pixmap%d",filename,i);
		gnome_config_set_string(buf,pos->pixmap_name);
	}
/*
 * These are all of the services and their commands
 */
 	for(i = 0, srv = service_list; srv; srv = srv->next, i++)
 	{
		sprintf(buf,"=%s=/global/service%d_name",filename,i);
		gnome_config_set_string(buf,srv->name);

		sprintf(buf,"=%s=/global/service%d_port",filename,i);
		gnome_config_set_int(buf,srv->port);
			
		sprintf(buf,"=%s=/global/service%d_protocol",filename,i);
		gnome_config_set_int(buf,srv->protocol);

		sprintf(buf,"=%s=/global/service%d_command",filename,i);
		gnome_config_set_string(buf,srv->command);
	}
	
	for(i=0, np = main_window->net_pages; np; np = np->next, i++)
	{
		sprintf(base,"=%s=/viewspace%d",filename,i);
		
		sprintf(buf,"%s%s",base, "/name");
		gnome_config_set_string(buf,np->name);
		
		/* add the reference to the page_objects in the netpage sections */
		strcat(base,"/page_object_");

		for(j = 0, po = np->page_objects; po; po = po->next, j++)
		{
			char *string;
			sprintf(buf, "%s%d",base,j);

			add_name_to_buf(buf, buf2, "_ip");
			gnome_config_set_string(buf2,ip2str(po->ip));

			add_name_to_buf(buf, buf2, "_name");
			gnome_config_set_string(buf2,po->name);
			
			add_name_to_buf(buf, buf2, "_x");
			gnome_config_set_int(buf2,po->x);
			
			add_name_to_buf(buf, buf2, "_y");
			gnome_config_set_int(buf2,po->y);

// There is no need to get the flags right now			
//			add_name_to_buf(buf, buf2, "_flags");
//			gnome_config_set_int(buf2,po->flags);
			
			if(po->notes)
			{
				string = page_object_convert_notes(po);
				add_name_to_buf(buf, buf2, "_notes");
				gnome_config_set_string(buf2,string);
				free(string);
			}
		}
	}
	
	gnome_config_sync();	
}

